#
# Copyright (C) 2019 Josefina Perles
#

"""
A script to load structures from CIF files into a CSD database
"""

import datetime
import locale
import os
import errno
import webbrowser

from ccdc import io
from ccdc.utilities import FileLogger
from mercury_interface import MercuryInterface

# get the absolute directory and the name without extension of this script
script_abs_path = os.path.abspath(__file__)
script_dir_name = os.path.dirname(script_abs_path)
script_name = os.path.splitext(os.path.basename(script_abs_path))[0]

# read options from configuration file
config_locals = {}
discard_globals = {}
try:
    execfile(os.path.join(script_dir_name,  script_name) + ".conf", discard_globals, config_locals)
except IOError:
    pass

# get options if provided in the configuration file, get default values otherwise
import_dir_name = config_locals.get("importdir", os.path.join(script_dir_name, "cif"))
database_path = config_locals.get("databasepath", os.path.join(script_dir_name, "db"))
database_base_name = config_locals.get("databasename", "sidi-scxrd")
database_file_name = os.path.join(database_path, database_base_name)

cif_file_list = []
if  os.path.isdir(import_dir_name):
    for dirpath, dirs, files in os.walk(import_dir_name):
        for filename in files:
            if filename.endswith(".cif"):
                cif_file_list.append(os.path.join(dirpath, filename))

now = datetime.datetime.today()

number_of_crystals = 0
if cif_file_list:
    try:
        os.makedirs(database_path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(database_path):
            pass
        else:
            raise

    with io.CrystalWriter(database_file_name + ".csdsql") as crystal_writer:
        for cif_file in cif_file_list:
            with io.CrystalReader(cif_file, format="cif") as crystal_reader:
                for cryst in crystal_reader:
                    crystal_writer.write(cryst)
                    number_of_crystals += 1
    if number_of_crystals:
        # os.rename(database_file_name + ".csdsql", database_file_name + "_ASER.sqlite")

        time_locale = locale.getlocale(locale.LC_TIME)
        locale.setlocale(locale.LC_TIME, "en_US")
        database_inf = ("#\n"
                        "# Database information file for ConQuest\n"
                        "#\n"
                        "{{\n"
                        # " 'title':                   'UAM SIdI SCXRD',\n"
                        " 'title':                   'In-house ({})',\n"
                        " 'date':                    '{}',\n"
                        " 'version':                 '0.{}',\n"
                        " 'list_priority':           300,\n"
                        "}}\n").format(database_base_name, now.strftime("%d %B %Y"), now.strftime("%y%m%d%H%M%S"))
        locale.setlocale(locale.LC_TIME, time_locale)

        with open(database_file_name + ".inf", "w") as dbini:
            dbini.writelines(database_inf)

with FileLogger(os.path.join(script_dir_name,  script_name) + ".log") as logger:
    logger.ignore_line_numbers(True)
    logger.info(now.isoformat(" "))
    logger.info("Searched {} directory for CIF files.".format(import_dir_name))
    logger.info("Found {} CIF files.".format(len(cif_file_list)))
    logger.info("Found {} crystals in the files.".format(number_of_crystals))
    logger.info("Crystals stored in the {} database.".format(database_file_name + ".csdsql"))

helper = MercuryInterface()
with open(helper.output_html_file, "w") as out:
    out.write("<!DOCTYPE html>")
    out.write("<html><head><title>Import CIF files int a CSD database</title></head>")
    out.write("<body>")
    out.write("<h1>CIF import script log</h1>")
    out.write("<p>Searched <strong>{}</strong> directory for CIF files.</p>".format(import_dir_name))
    out.write("<p>Found <strong>{}</strong> CIF files.</p>".format(len(cif_file_list)))
    out.write("<p>Found <strong>{}</strong> crystals in the files.</p>".format(number_of_crystals))
    out.write("<p>Crystals stored in the <strong>{}</strong> database.</p>".format(database_file_name + ".csdsql"))
    out.write("</body></html>")
