#
# This script can be used for any purpose without limitation subject to the
# conditions at http://www.ccdc.cam.ac.uk/Community/Pages/Licences/v2.aspx
#
# This permission notice and the following statement of attribution must be
# included in all copies or substantial portions of this script.
#
# 2015-05-01: created by Clare Macrae, the Cambridge Crystallographic Data Centre
# 2015-06-18: made available by the Cambridge Crystallographic Data Centre
#

'''
    mercury_interface.py   -   parse arguments sent by mercury

This file is designed to match the version of Mercury it is released with,
and so it is not intended to be modified by users.

'''
import sys
import json
import os
import shutil

import ccdc.io

import warnings
# Warnings will cause the launched script to crash in mercury, so let's ignore them.
warnings.simplefilter('ignore')


class MercuryInterface(object):
    '''Argument parser for arguments sent by mercury.

    Supports both old-style refcode and html file, and new-style refcode, html, json-encoded dictionary.
    '''
    def __init__(self):
        if len(sys.argv) < 3:
            raise RuntimeError("Insufficient Arguments")
        elif len(sys.argv) == 3:
            # Old-style parameters, as sent from Mercury 3.5, which only worked on CSD databases
            self.identifier = sys.argv[1]
            self.file_path = ""
            self.output_html_file = sys.argv[2]
            self.output_csv_file = ""
            self.output_tsv_file = ""
        else:
            m2a_path = sys.argv[3]
            base, ext = os.path.splitext(m2a_path)

            if ext == ".h2a":
                raise RuntimeError("Got Hermes-to-API interchange file. Are you running a Mercury script with Hermes?")
            if ext != ".m2a":
                raise RuntimeError("Expected Mercury-to-API interchange file.")

            with file(m2a_path) as m2a_reader:
                # The content here is defined by main/guilib/guilib_qt/ScriptRunnerRunInfo.cpp
                text = m2a_reader.read()

            data = json.loads(text)
            self.options = data['options']
            self.identifier = self.options['identifier']
            self.file_path = self.options['database_path']
            self.program_path = self.options['program_executable_path']
            self.conquest_path = self.options['conquest_path']
            self.output_directory_path = self.options['working_directory_path']
            self.output_base = self.options['output_base']
            self.output_html_file = self.output_base + ".html"
            self.output_csv_file = self.output_base + ".csv"
            self.output_tsv_file = self.output_base + ".tsv"
            self.output_c2m_file = self.output_base + ".c2m"
            self.output_gcd_file = self.output_base + ".gcd"
            self.output_morphology_file = self.output_base + "_morphology.cif"

    @property
    def log_file_name(self):
        '''The log file to be used.'''
        return self.output_base + '.log'

    @property
    def current_entry(self):
        '''The entry of the given identifier.'''
        reader = ccdc.io.EntryReader(self.file_path)
        return reader.entry(self.identifier)

    @property
    def selected_atoms(self):
        '''The list of selected atoms.'''
        mol = self.current_entry.molecule
        return [a for a in mol.atoms if a.index in self.options['selected_atoms']]

    @property
    def selected_indices(self):
        '''The list of atom indices requested.'''
        return self.options['selected_atoms']

    @property
    def distances(self):
        '''The list of pairs defining requested distances.'''
        return [self.options['distances'][i:i+2] for i in range(0, len(self.options['distances']), 2)]

    @property
    def angles(self):
        '''The list of triples defining requested angles.'''
        return [self.options['angles'][i:i+3] for i in range(0, len(self.options['angles']), 3)]

    @property
    def torsion_angles(self):
        '''The list of 4-tuples defining requested torsion angles.'''
        return [self.options['torsions'][i:i+4] for i in range(0, len(self.options['torsions']), 4)]

    @property
    def get_ccdc_logo(self):
        '''The filename of a CCDC logo that can used in output HTML files

        This copies a logo from the Mercury distribution to the output directory.
        The alternative approach, of returning a URL, has a given broken image when running Mercury
        in environments with proxy servers.
        '''
        filename = "ccdc_logo_180x180_with_text.png"
        filename_in_output_directory = os.path.join(self.output_directory_path, filename)
        searched_locations = []
        if not os.path.exists(filename_in_output_directory):
            possible_relative_paths_to_logo = [
                "images/",
                "../images/",
                "../../images/",
                "../../cppbuilds_shared/installer/images/",
                "../../../cppbuilds_shared/installer/images/",
                "../../../../cppbuilds_shared/installer/images/",
            ]
            program_directory = os.path.split(self.program_path)[0]
            for relative_path in possible_relative_paths_to_logo:
                potential_path = os.path.join(program_directory, relative_path, filename)
                searched_locations.append(potential_path)
                if os.path.exists(potential_path):
                    shutil.copyfile(potential_path, filename_in_output_directory)
                    return filename
            if 'MAINDIR' in os.environ:
                build_space_path = os.path.join(os.environ['MAINDIR'], "cppbuilds_shared/installer/images/", filename)
                if os.path.exists(build_space_path):
                    shutil.copyfile(build_space_path, filename_in_output_directory)
                    return filename

        raise RuntimeError('Cannot find logo: searched these locations' + str('\n'.join(searched_locations)))

    def show_script_error(self, message):
        """Create a user-friendly error report

        :param message: Text of error message to display
        """
        with open(self.output_html_file, "w") as out:
            out.write('<html>')
            out.write('<h1>Script Error</h1>')
            out.write('<p><b>{}</b></p>'.format(message))
            out.write('<p>Script launched by: {}</p>'.format(self.program_path))
            out.write('<p>CSD refcode: {}</p>'.format(self.identifier))
            out.write('<p>current directory: {}</p>'.format(os.getcwd()))
            out.write('</html>')

    def open_output_folder(self):
        '''Open the output folder in the computers file browser.
        '''

        # Implementation based upon:
        # http://stackoverflow.com/questions/6631299/python-opening-a-folder-in-explorer-nautilus-mac-thingie
        import webbrowser
        webbrowser.open(self.output_directory_path)

if __name__ == "__main__":
    raise RuntimeError('\n\nSorry - this is a helper script.\n'
                       'It is not intended for running - and should somehow '
                       'be hidden from the scripts menu')
